
/* config.c -- configuration manager based on libPropList.

 * Copyright (C) 1999 Yarick Rastrigin <yarick@relex.ru>
 *                    Steve Hill <sjhill@plutonium.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <glib.h>
#include <proplist.h>

#include "common.h"
#include "make_dir_list.h"
#include "clipboard.h"
#include "pixmaps.h"
#include "config.h"

config_item config[] =
{
  {
    "main_window_width",
    "500",
    get_main_window_width,
    set_main_window_width
  },
  {
    "main_window_height",
    "400",
    get_main_window_height,
    set_main_window_height
  },
  {
    "main_treewin_width",
    "300",
    get_main_treewin_width,
    set_main_treewin_width,
  },
  {
    "main_treewin_height",
    "400",
    get_main_treewin_height,
    set_main_treewin_height,
  },
  {
    "dir_list_mode",
    "2",
    get_dir_list_mode,
    set_dir_list_mode,
  },
  {
    "sort_mode",
    "0",
    get_dir_list_sort_mode,
    set_dir_list_sort_mode
  },
  {
    "show_hidden",
    "TRUE",
    get_hidden_state,
    set_hidden_state
  },
  {
    "show_backup",
    "TRUE",
    get_backup_state,
    set_backup_state
  }
};

GList *gfb_typedefs = NULL;
//typedef char *(*GetValueFunc) (void);
//typedef void (*SetValueFunc) (char *);


void
load_config (gchar * filename)
{
  proplist_t hpt1 = NULL;
  proplist_t hpt2 = NULL;
  proplist_t hpt3 = NULL;
  gint hgi1, hgi2;
  gchar *hgc1;
  hgi1 = sizeof (config) / sizeof (config_item);
  hpt1 = PLGetProplistWithPath (filename);
  for (hgi2 = 0; hgi2 < hgi1; hgi2++)
    {
      hpt2 = PLMakeString (config[hgi2].name);
      if (hpt1 != NULL)
	{
	  hpt3 = PLGetDictionaryEntry (hpt1, hpt2);
	  if (hpt3 != NULL)
	    hgc1 = PLGetString (hpt3);
	  else
	    hgc1 = config[hgi2].default_value;
	}
      else
	hgc1 = config[hgi2].default_value;
      config[hgi2].set_func (hgc1);
      PLRelease (hpt2);
      if (hpt3 != NULL)
	PLRelease (hpt3);
    };
}

void
save_config (gchar * filename)
{
  proplist_t hpt1 = NULL;
  proplist_t hpt2 = NULL;
  proplist_t hpt3 = NULL;
  gint hgi1, hgi2;
  hgi1 = sizeof (config) / sizeof (config_item);
  /* F...ing libPropList don't allows creation of empty dictionary, so we must
     process first parameter separately.
   */
  hpt2 = PLMakeString (config[0].name);
  hpt3 = PLMakeString (config[0].get_func ());
  hpt1 = PLMakeDictionaryFromEntries (hpt2, hpt3, NULL);
  PLRelease (hpt2);
  PLRelease (hpt3);
  for (hgi2 = 1; hgi2 < hgi1; hgi2++)
    {
      hpt2 = PLMakeString (config[hgi2].name);
      hpt3 = PLMakeString (config[hgi2].get_func ());
      hpt1 = PLInsertDictionaryEntry (hpt1, hpt2, hpt3);
      PLRelease (hpt2);
      PLRelease (hpt3);
    };
  hpt2 = PLMakeString (filename);
  hpt1 = PLSetFilename (hpt1, hpt2);
  PLSave (hpt1, NO);
  PLRelease (hpt2);
  PLRelease (hpt1);
}

void
load_typedata (gchar * filename)
{
  gint hgi1, hgi2, hgi3, hgi4;
  proplist_t hpt1, hpt2, hpt3, hpt4, hpt5;
  proplist_t Typedef, Type, Operations;
  GfbTypeDef *hGTD;
  GfbItemAction *hGIA;
  GList *hGL1 = gfb_typedefs;
  hpt1 = PLGetProplistWithPath (filename);
  if (hpt1 == NULL)
    {
#ifdef __DEBUG__
      debug_out ("Check your typedefs file (~/.gfb/gfbtypes)");
#endif /* __DEBUG__ */
      return;
    };

  hpt2 = PLMakeString ("TypeDefs");
  hpt3 = PLGetDictionaryEntry (hpt1, hpt2);
  PLRelease (hpt2);
  hgi1 = PLGetNumberOfElements (hpt3);
  for (hgi2 = 0; hgi2 < hgi1; hgi2++)
    {
      Typedef = PLGetArrayElement (hpt3, hgi2);
      hpt2 = PLMakeString ("Type");
      Type = PLGetDictionaryEntry (Typedef, hpt2);
      hGTD = g_malloc0 (sizeof (GfbTypeDef));
      hGTD->name = g_strdup (PLGetString (Type));
      PLRelease (hpt2);
      hpt2 = PLMakeString ("Operations");
      Operations = PLGetDictionaryEntry (Typedef, hpt2);
      PLRelease (hpt2);
      hgi3 = PLGetNumberOfElements (Operations);
      for (hgi4 = 0; hgi4 < hgi3; hgi4++)
	{
	  hpt4 = PLGetArrayElement (Operations, hgi4);
	  hGIA = g_malloc0 (sizeof (GfbItemAction));
	  hGIA->name = g_strdup (PLGetString (hpt4));
	  hpt2 = PLMakeString (hGIA->name);
	  hpt5 = PLGetDictionaryEntry (Typedef, hpt4);
	  PLRelease (hpt2);
	  hGIA->action = g_strdup (PLGetString (hpt5));
	  hGTD->actions = g_list_append (hGTD->actions, hGIA);
	};
      gfb_typedefs = g_list_append (gfb_typedefs, hGTD);
    };
  PLRelease (hpt1);
}

void
save_typedata (gchar * filename)
{
  /*
     So we need: (a brief explanation of algorithm)
     1. Create an array of dictionaries
     a) For every GfbTypeDef create dictionary containing Key-Value pairs
     a1) Create an array "Operations" and put there all operations basing on GfbItemAction lists;
     a2) walk through every operation and create key-value pairs
     b) Store dictionary for later PLMakeArrayFromElements();
   */
  GList *hGL1, *hGL2, *hGL3;
  proplist_t hpt1, hpt2, hpt3, hpt4, hpt5;
  gint hgi1, hgi2, hgi3, hgi4;
  gchar *hgc1;
  hGL1 = gfb_typedefs;
  hpt1 = PLMakeArrayFromElements (NULL);	/* Get ready for adding */
  while (hGL1 != NULL)
    {
      hGL2 = NULL;
      hpt2 = PLMakeArrayFromElements (NULL);	/* For Operations */
      hGL3 = ((GfbTypeDef *) (hGL1->data))->actions;
      hpt3 = PLMakeString ("Type");
      hpt4 = PLMakeString (((GfbTypeDef *) (hGL1->data))->name);
      hpt5 = PLMakeDictionaryFromEntries (hpt3, hpt4, NULL);
      while (hGL3 != NULL)
	{
	  hpt3 = PLMakeString (((GfbItemAction *) (hGL3->data))->name);
	  hpt4 = PLMakeString (((GfbItemAction *) (hGL3->data))->action);
	  hpt5 = PLInsertDictionaryEntry (hpt5, hpt3, hpt4);
	  hpt2 = PLAppendArrayElement (hpt2, hpt3);	/* Inserting an Operation reference */
	  hGL3 = hGL3->next;
	};
      hpt3 = PLMakeString ("Operations");
      hpt5 = PLInsertDictionaryEntry (hpt5, hpt3, hpt2);	/* Inserting Operations array */
      hpt1 = PLAppendArrayElement (hpt1, hpt5);
      hGL1 = hGL1->next;
    };
  hpt3 = PLMakeString ("TypeDefs");
  hpt5 = PLMakeDictionaryFromEntries (hpt3, hpt1, NULL);	/* Main array created */
  hpt4 = PLMakeString (filename);
  PLSetFilename (hpt5, hpt4);
  PLSave (hpt5, YES);
}


void *
find_typedef (gchar * type_def)
{
  GList *hGL1 = gfb_typedefs;
  g_return_val_if_fail(type_def != NULL, NULL);
  while (hGL1 != NULL)
    {
      if (!strcmp (GFB_TYPE_DEF (hGL1->data)->name, type_def))
	return hGL1->data;
      hGL1 = hGL1->next;
    };
  return NULL;
};

void *
find_typedef_by_substr (gchar * type_def)
{
  gchar *hgc1, *hgc2, *hgc3;
  gboolean done;
  GList *hGL1 = gfb_typedefs;
  g_return_val_if_fail(type_def != NULL, NULL);
  while (hGL1 != NULL)
    {
      hgc1 = g_strdup(GFB_TYPE_DEF(hGL1->data)->name);
      hgc3 = hgc1; hgc2 = hgc1;done = FALSE;
      while ( !done )
        {
            while ((*hgc3 != '\0') && (*hgc3 != '|'))
                hgc3++;
            if (*hgc3 == '\0') done = TRUE;
            *hgc3 = '\0'; hgc3++;
            if (strstr (type_def, hgc2))
                {
                    g_free(hgc1);
                    return hGL1->data;
                };
            hgc2 = hgc3;
        };
	  g_free(hgc1);
      hGL1 = hGL1->next;
    };
  return NULL;
};

gchar *
find_action (void *type_def, gchar * action_name)
{
  GList *hGL1 = GFB_TYPE_DEF (type_def)->actions;
  while (hGL1 != NULL)
    {
      if (!strcmp (((GfbItemAction *) (hGL1->data))->name, action_name))
	return ((GfbItemAction *) (hGL1->data))->action;
      hGL1 = hGL1->next;
    };
  return NULL;
}


